import re


class StripTag:

    def __init__(self, tag_name):
        tag_name = tag_name.encode('utf-8')
        self.replacement = b'<%s>' % tag_name
        self.rule = re.compile(b'<%s\\s[^>]+>' % tag_name)

    async def after_response(self, entry):
        entry.response.raw = self.rule.sub(self.replacement, entry.response.raw)



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

import re
from urllib.parse import urljoin
from urllib.parse import urlparse

from hammertime.ruleset import StopRequest, RejectRequest

from tachyon import conf, textutils, database


def add_path(path):
    current_template = conf.path_template.copy()
    current_template['description'] = 'Found in sitemap.xml'
    current_template['is_file'] = False
    current_template['url'] = '/' + path
    if current_template not in database.paths:
        database.paths.append(current_template)
        return True


def add_file(filename):
    """ Add file to database """
    current_template = conf.path_template.copy()
    current_template['description'] = 'Found in sitemap.xml'
    current_template['url'] = filename
    if current_template not in database.files:
        database.files.append(current_template)
        return True


async def execute(hammertime):
    """ Fetch sitemap.xml and add each entry as a target """

    current_template = dict(conf.path_template)
    current_template['description'] = 'sitemap.xml entry'

    target_url = urljoin(conf.base_url, "/sitemap.xml")

    try:
        entry = await hammertime.request(target_url)

        regexp = re.compile('(?im).*<url>\\s*<loc>(.*)</loc>\\s*</url>.*')
        matches = re.findall(regexp, entry.response.content)

        added = 0
        for match in matches:
            if not isinstance(match, str):
                match = match.decode('utf-8', 'ignore')
            parsed = urlparse(match)
            if parsed.path:
                new_path = parsed.path
            else:
                continue

            # Remove trailing /
            if new_path.endswith('/'):
                new_path = new_path[:-1]

            if add_path(new_path):
                added += 1

        if added > 0:
            textutils.output_info(' - SitemapXML Plugin: added %d base paths '
                                  'using /sitemap.xml' % added)
        else:
            textutils.output_info(' - SitemapXML Plugin: no usable entries '
                                  'in /sitemap.xml')
    except (StopRequest, RejectRequest):
        textutils.output_info(' - SitemapXML Plugin: /sitemap.xml not found on '
                              'target site')



import asyncio
import json
import uuid
from os.path import join

from hammertime.utils.har import HammerTimeToHAR
from marshmallow_har import HAR, Creator
from tachyon.__version__ import __version__


class FileWriter:

    def __init__(self, output_dir):
        self.dir = output_dir

    def __call__(self, har: HAR):
        filename = "%s.har" % uuid.uuid4()
        file_path = join(self.dir, filename)
        with open(file_path, "w") as fp:
            fp.write(json.dumps(har.dump().data, indent=4))
        return file_path


class StoreHAR:

    def __init__(self, writer):
        self.writer = writer
        self.converter = HammerTimeToHAR()

    async def on_request_successful(self, entry):
        await asyncio.get_event_loop().run_in_executor(None, self._write_har, entry)

    def _write_har(self, entry):
        har = self.converter.convert_entries([entry], creator=Creator(name="Tachyon", version=__version__))
        entry.result.har_location = self.writer(har)



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA


import tachyon.database as database


class PathGenerator:

    def generate_paths(self, *, use_valid_paths):
        generated_paths = []
        if use_valid_paths:
            generated_paths.extend(self._create_new_paths_from_valid_paths())
            database.path_cache.update(path["url"] for path in generated_paths)
        else:
            generated_paths.extend([path for path in self._database_paths()])
            generated_paths.extend([file for file in self._use_files_as_paths()])
        return generated_paths

    def _database_paths(self):
        for path in database.paths:
            if path["url"] not in database.path_cache:
                database.path_cache.add(path["url"])
                yield path

    def _use_files_as_paths(self):
        for file in database.files:
            path = "/%s" % file["url"]
            if not file.get("no_suffix") and path not in database.path_cache:
                file_as_path = file.copy()
                file_as_path["url"] = path
                database.path_cache.add(path)
                yield file_as_path

    def _create_new_paths_from_valid_paths(self):
        new_paths = []
        for path in database.valid_paths:
            for _path in database.paths:
                new_path = self._join_paths(path, _path)
                if new_path is not None and new_path["url"] not in database.path_cache:
                    new_paths.append(new_path)
        return new_paths

    def _join_paths(self, leading_path, trailing_path):
        if leading_path["url"] != "/" and trailing_path["url"] != "/":
            new_path = trailing_path.copy()
            new_path["url"] = leading_path["url"] + trailing_path["url"]
            return new_path
        return None


class FileGenerator:

    def __init__(self):
        self.file_suffixes = ['', '.sql', '.bak', '-bak', '.old', '-old', '.dmp', '.dump', '.zip', '.rar', '.7z',
                              '.tar.gz', '.tar.bz2', '.tar', '.tgz', '~', '.conf.old', '.conf', '.config', '.conf.orig',
                              '.conf.bak', '.cnf', '.cfg', '.ini', '.inc', '.inc.old', '.inc.orig', '.log', '.txt',
                              '_log', '.passwd', '.php.bak', '.php.old', '.php.inc', '.php.orig', '.sql.old',
                              '.sql.bak', '0', '1', '2', '.xml', '.csv', '.wsdl', '.pwd', '.yml']
        self.executables_suffixes = ['.php', '.asp', '.aspx', '.pl', '.cgi', '.cfm']

    def generate_files(self, skip_root=False):
        files = list()
        for path in database.valid_paths:
            if not skip_root or not self._is_root(path):
                files.extend(self._add_all_possible_files_to_path(path))
        return files

    def _is_root(self, path):
        if path == "/":
            return True
        elif isinstance(path, dict):
            return self._is_root(path["url"])
        else:
            return False

    def _add_all_possible_files_to_path(self, path):
        for file in database.files:
            if file.get('no_suffix'):
                new_filename = self._create_file(file, path, is_file=True, no_suffix=True)
                yield new_filename
            elif file.get('executable'):
                yield from self._create_executable_files(path, file)
            else:
                yield from self._create_files_with_suffixe(path, file)

    def _create_executable_files(self, path, file):
        for executable_suffix in self.executables_suffixes:
            new_filename = self._create_file(file, path, suffix=executable_suffix, is_file=True, executable=True)
            yield new_filename

    def _create_files_with_suffixe(self, path, file):
        for suffix in self.file_suffixes:
            new_filename = self._create_file(file, path, suffix=suffix, is_file=True)
            yield new_filename

    def _create_file(self, base_file, path, suffix="", **kwargs):
        url = self._join_path(path["url"], base_file["url"] + suffix)
        file = base_file.copy()
        file.update(url=url, **kwargs)
        return file

    def _join_path(self, base_path, file_path):
        if base_path == "/":
            return"/{file}".format(file=file_path.strip("/"))
        else:
            return "/{path}/{file}".format(path=base_path.strip("/"), file=file_path.strip("/"))



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA


from aiohttp import ClientSession, TCPConnector
from aiohttp.cookiejar import DummyCookieJar
from hammertime import HammerTime
from hammertime.config import custom_event_loop
from hammertime.engine import AioHttpEngine
from hammertime.engine.scaling import SlowStartPolicy, StaticPolicy
from hammertime.kb import KnowledgeBase
from hammertime.ruleset import StopRequest
from hammertime.rules.sampling import ContentHashSampling, ContentSampling, ContentSimhashSampling
from hammertime.rules.waf import RejectWebApplicationFirewall
from hammertime.rules import DetectSoft404, RejectStatusCode, DynamicTimeout, RejectCatchAllRedirect, FollowRedirects, \
    SetHeader, DeadHostDetection, FilterRequestFromURL, DetectBehaviorChange, IgnoreLargeBody, RedirectLimiter

from tachyon import conf
from tachyon.heuristics import RejectIgnoredQuery, LogBehaviorChange, MatchString, StripTag
from tachyon.filefetcher import ValidateEntry

heuristics_with_child = []
initial_limit = 5120
default_user_agent = 'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) ' \
                     'Chrome/41.0.2228.0 Safari/537.36'


async def configure_hammertime(proxy=None, retry_count=3, cookies=None, concurrency=0, **kwargs):
    loop = custom_event_loop()
    engine = AioHttpEngine(loop=loop, verify_ssl=False, proxy=proxy)
    await engine.session.close()
    connector = TCPConnector(loop=loop, verify_ssl=False, use_dns_cache=True, ttl_dns_cache=None)
    if cookies is not None:
        engine.session = ClientSession(loop=loop, connector=connector, cookie_jar=DummyCookieJar(loop=loop))
    else:
        engine.session = ClientSession(loop=loop, connector=connector)

    scale_policy = SlowStartPolicy(initial=3)
    if concurrency > 0:
        scale_policy = StaticPolicy(concurrency)

    kb = KnowledgeBase()
    hammertime = HammerTime(loop=loop, request_engine=engine, retry_count=retry_count, proxy=proxy, kb=kb,
                            scale_policy=scale_policy)
    setup_hammertime_heuristics(hammertime, **kwargs)
    hammertime.collect_successful_requests()
    hammertime.kb = kb
    return hammertime


def setup_hammertime_heuristics(hammertime, *,
                                user_agent=default_user_agent, vhost=None, confirmation_factor=1,
                                har_output_dir=None):
    global heuristics_with_child
    dead_host_detection = DeadHostDetection(threshold=200)
    detect_soft_404 = DetectSoft404(distance_threshold=6, confirmation_factor=confirmation_factor)
    follow_redirects = FollowRedirects()
    heuristics_with_child = [RejectCatchAllRedirect(), follow_redirects,
                             RejectIgnoredQuery()]
    hosts = (vhost, conf.target_host) if vhost is not None else conf.target_host

    init_heuristics = [SetHeader("User-Agent", user_agent),
                       SetHeader("Host", vhost if vhost is not None else conf.target_host),
                       ContentHashSampling(), ContentSampling(), ContentSimhashSampling(),
                       dead_host_detection,
                       RejectStatusCode({503, 508}, exception_class=StopRequest),
                       StripTag('input'), StripTag('script')]

    global_heuristics = [RejectStatusCode({404, 406, 502}),
                         RejectWebApplicationFirewall(),
                         DynamicTimeout(1.0, 5),
                         RedirectLimiter(),
                         FilterRequestFromURL(allowed_urls=hosts),
                         IgnoreLargeBody(initial_limit=initial_limit)]

    # Dead host detection must be first to make sure there is no skipped after_headers
    hammertime.heuristics.add_multiple(init_heuristics)

    # General
    hammertime.heuristics.add_multiple(global_heuristics)
    hammertime.heuristics.add_multiple(heuristics_with_child)
    hammertime.heuristics.add_multiple([
        detect_soft_404,
        MatchString(),
        ValidateEntry(),
        DetectBehaviorChange(buffer_size=100),
        LogBehaviorChange(),
        ValidateEntry(),
    ])
    detect_soft_404.child_heuristics.add_multiple(init_heuristics)
    detect_soft_404.child_heuristics.add_multiple(heuristics_with_child)

    for heuristic in heuristics_with_child:
        heuristic.child_heuristics.add_multiple(init_heuristics)
        heuristic.child_heuristics.add_multiple(global_heuristics)

    if har_output_dir is not None:
        from tachyon.har import StoreHAR, FileWriter
        hammertime.heuristics.add(StoreHAR(writer=FileWriter(har_output_dir)))


def add_http_header(hammertime, header_name, header_value):
    set_header = SetHeader(header_name, header_value)
    hammertime.heuristics.add(set_header)
    for heuristic in heuristics_with_child:
        heuristic.child_heuristics.add(set_header)


def set_cookies(hammertime, cookies):
    add_http_header(hammertime, "Cookie", cookies)



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

from tachyon import database


def _get_cached_url_string(url_obj):
    if len(url_obj['url']) == 1 and url_obj['url'] == '/':
        return url_obj['url']

    return url_obj['url'].strip('/')


def add_path_to_fetch_queue(url_obj):
    """
     Add a path to the fetch queue but makes sure it's not already there.
     returns True if the path was not in the list, False if it's a duplicate
    """
    url_string = _get_cached_url_string(url_obj)
    if url_string not in database.path_cache:
        database.path_cache.add(url_string)
        return True
    else:
        return False


def add_file_to_fetch_queue(url_obj):
    """
     Add a file to the fetch queue but makes sure it's not already there.
     returns True if the file was not in the list, False if it's a duplicate
    """
    url_string = _get_cached_url_string(url_obj)
    if url_string not in database.file_cache:
        database.file_cache.add(url_string)
        return True
    else:
        return False



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from hammertime.ruleset import StopRequest, RejectRequest
from urllib.parse import urljoin

from tachyon import conf, textutils


base_headers = dict()


def save_file(path, content):
    output = "output/" + conf.target_host + path

    if not os.path.exists(output[:output.rfind('/')]):
        os.makedirs(output[:output.rfind('/')])

    with open(output, "wb") as outfile:
        if isinstance(content, str):
            outfile.write(content.encode('utf-8'))
        else:
            outfile.write(content)

# Fixme
#def parse_svn_17_db(filename):
#    conn = sqlite3.connect(filename)
#    files = conn.execute('select local_relpath, ".svn/pristine/" || substr(checksum,7,2) || "/" || substr(checksum,7) || ".svn-base" as alpha from NODES;')
#    #for entries in files:
#    #    if
#    pass


async def parse_svn_entries(url, hammertime):
    description_file = 'SVN entries file at'
    description_dir = "SVN entries Dir at"
    target_url = url + "/.svn/entries"

    try:
        entry = await hammertime.request(target_url)
        tokens = entry.response.content.split('\n')
        if 'dir' in tokens:
            for pos, token in enumerate(tokens):
                if token == 'dir':
                    # Fetch more entries recursively
                    if tokens[pos-1] != '':
                        if conf.allow_download:
                            textutils.output_info(' - Svn Plugin: Downloading: ' + url + '/' + tokens[pos - 1] + '\r')
                        else:
                            textutils.output_found(description_dir + ' at: ' + url + '/' + tokens[pos - 1])

                        # Parse next
                        await parse_svn_entries(url + "/" + tokens[pos-1], hammertime)

                elif token == 'file':
                    if conf.allow_download:
                        textutils.output_info(' - Svn Plugin: Downloading: ' + url + '/' + tokens[pos - 1] + '\r')
                        # Fetch text-base file
                        path = url + "/.svn/text-base" + '/' + tokens[pos-1] + ".svn-base"
                        entry = await hammertime.request(path)
                        save_file(url + '/' + tokens[pos-1], entry.response.content)
                    else:
                        textutils.output_found(description_file + ' at: ' + url + '/' + tokens[pos - 1])
    except (RejectRequest, StopRequest):
        pass


async def execute(hammertime):
    """ Fetch /.svn/entries and parse for target paths """

    textutils.output_info(' - Svn Plugin: Searching for /.svn/entries')
    target_url = urljoin(conf.base_url, "/.svn/entries")
    svn_legacy = True

    try:
        entry = await hammertime.request(target_url)
        if conf.allow_download:
            textutils.output_info(' - Svn Plugin: /.svn/entries found! crawling... (will download files to output/)')
        else:
            textutils.output_info(' - Svn Plugin: /.svn/entries found! crawling... (use -a to download files instead of printing)')
       
        # test for version 1.7+
        target_url = urljoin(conf.base_url, "/.svn/wc.db")
        entry = await hammertime.request(target_url)

        #if response_code in conf.expected_file_responses and content:
        #    textutils.output_info(' - Svn Plugin: SVN 1.7+ detected, parsing wc.db')
        #    svn_legacy = False
        #    save_file(conf.target_base_path + '/wc.db', content)

        # Process index
        if svn_legacy:
            # parse entries
            await parse_svn_entries(conf.base_url, hammertime)
        #else:
          #  parse_svn_17_db(conf.target_base_path + '/wc.db')

        # Clean up display
        if conf.allow_download:
            textutils.output_info('')
    except (StopRequest, RejectRequest):
        textutils.output_info(' - Svn Plugin: no /.svn/entries found')



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

import re
from urllib.parse import urljoin

from hammertime.ruleset import StopRequest, RejectRequest

from tachyon import conf, textutils, database


async def execute(hammertime):
    """ Fetch /robots.txt and add the disallowed paths as target """
    current_template = dict(conf.path_template)
    current_template['description'] = 'Robots.txt entry'

    target_url = urljoin(conf.base_url, "/robots.txt")

    try:
        entry = await hammertime.request(target_url)
        matches = re.findall(r'Disallow:\s*/[a-zA-Z0-9-/\r]+\n', entry.response.content)

        added = 0
        for match in matches:
            # Filter out some characters
            match = filter(lambda c: c not in ' *?.\n\r\t', match)

            if match:
                match = ''.join(match)

            # Split on ':'
            splitted = match.split(':')
            if splitted[1]:
                target_path = splitted[1]

                # Remove trailing /
                if target_path.endswith('/'):
                    target_path = target_path[:-1]

                current_template = current_template.copy()
                current_template['url'] = target_path
                database.paths.append(current_template)
                added += 1

        if added > 0:
            textutils.output_info(' - Robots Plugin: added ' + str(added) + ' base paths using /robots.txt')
        else:
            textutils.output_info(' - Robots Plugin: no usable entries in /robots.txt')
    except (StopRequest, RejectRequest):
        textutils.output_info(' - Robots Plugin: /robots.txt not found on target site')



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

from datetime import date

from tachyon import conf, textutils, database


plugin_settings = conf.plugin_settings["PathGenerator"]


def add_generated_path(path):
    current_template = conf.path_template.copy()
    current_template['description'] = 'Computer generated path'
    current_template['is_file'] = False
    current_template['url'] = '/' + path
    current_template['handle_redirect'] = "ignoreRedirect" not in plugin_settings
    database.paths.append(current_template)


def add_generated_file(file):
    """ Add file to database """
    current_template = conf.path_template.copy()
    current_template['description'] = 'Computer generated file'
    current_template['url'] = file
    current_template['handle_redirect'] = "ignoreRedirect" not in plugin_settings
    database.files.append(current_template)


async def execute(hammertime):
    """ Generate common simple paths (a-z, 0-9) """
    path_added = 0
    file_added = 0

    if "skipAlpha" not in plugin_settings:
        for char in range(ord('a'), ord('z')+1):
            add_generated_path(chr(char))
            path_added += 1
            add_generated_file(chr(char))
            file_added += 1

    if "skipNumeric" not in plugin_settings:
        for char in range(ord('0'), ord('9')+1):
            add_generated_path(chr(char))
            path_added += 1
            add_generated_file(chr(char))
            file_added += 1

    if "skipYear" not in plugin_settings:
        for year in range(1990, date.today().year + 5):
            add_generated_path(str(year))
            path_added += 1

    textutils.output_info(' - PathGenerator Plugin: added ' + str(path_added) + ' computer generated path.')
    textutils.output_info(' - PathGenerator Plugin: added ' + str(file_added) + ' computer generated files.')



from setuptools import setup, find_packages

from tachyon.__version__ import __version__


setup(
    name='tachyon3',
    url="https://github.com/delvelabs/tachyon",
    version=__version__,
    packages=find_packages(),
    python_requires='>=3.6.0,<3.9.0',
    package_data={'tachyon': ['data/*.json']},
    entry_points={
        'console_scripts': [
            'tachyon = tachyon.__main__:main'
        ]
    },
    install_requires=[
        'hammertime-http[simhash-py]>=0.8,<0.9',
        'easyinject==0.3',
        'click==7.1.2'
    ],
)



# Tachyon - Fast Multi-Threaded Web Discovery Tool
# Copyright (c) 2011 Gabriel Tremblay - initnull hat gmail.com
# Copyright (C) 2018-  Delve Labs inc.
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA


from tachyon.textutils import output_info


class LogBehaviorChange:

    MESSAGE = "Behavior change detected! The above results are known to be unreliable. " \
              "Re-validation will occur at the end of the scan."

    def __init__(self):
        self.has_error = False

    async def on_request_successful(self, entry):
        if not self.has_error and entry.result.error_behavior:
            self.has_error = True
            output_info(self.MESSAGE)
